"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .settlement import Settlement, SettlementTypedDict
from gr4vy.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class SettlementsTypedDict(TypedDict):
    r"""A list of settlement records for a transaction."""

    items: List[SettlementTypedDict]
    r"""The list of settlement objects."""


class Settlements(BaseModel):
    r"""A list of settlement records for a transaction."""

    items: List[Settlement]
    r"""The list of settlement objects."""
