"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel
from gr4vy.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal, Optional
from typing_extensions import Annotated, TypedDict


class CardSchemeDefinitionTypedDict(TypedDict):
    id: str
    r"""The ID for the card scheme."""
    icon_url: str
    r"""The icon for this card scheme."""
    display_name: str
    r"""The display name of this card scheme."""
    type: Literal["card-scheme-definition"]
    r"""Always `card-scheme-definition`."""


class CardSchemeDefinition(BaseModel):
    id: str
    r"""The ID for the card scheme."""

    icon_url: str
    r"""The icon for this card scheme."""

    display_name: str
    r"""The display name of this card scheme."""

    TYPE: Annotated[
        Annotated[
            Optional[Literal["card-scheme-definition"]],
            AfterValidator(validate_const("card-scheme-definition")),
        ],
        pydantic.Field(alias="type"),
    ] = "card-scheme-definition"
    r"""Always `card-scheme-definition`."""
