"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel
from typing_extensions import TypedDict


class CreatorTypedDict(TypedDict):
    id: str
    name: str
    email_address: str


class Creator(BaseModel):
    id: str

    name: str

    email_address: str
